from django.urls import re_path
from django.conf import settings
from django.contrib import admin
from django.views.static import serve


def define_pth():
    urls_prefix = settings.PATH_PREFIX or ''
    if urls_prefix:
        urls_prefix += '/'
    media_root = {'document_root': settings.MEDIA_ROOT}
    media_urls = [re_path(r'^' + urls_prefix + 'media/(?P<path>.*)$', serve, media_root)]

    static_root = {'document_root': settings.STATIC_ROOT}
    static_urls = [re_path(r'^' + urls_prefix + 'static/(?P<path>.*)$', serve, static_root)]

    # admin.autodiscover()
    if urls_prefix:
        urls_prefix = r'^'+urls_prefix
    app_urls = [
        re_path(urls_prefix + 'admin/', admin.site.urls),
    ]
    all_urls = media_urls + static_urls + app_urls
    return all_urls


urlpatterns = define_pth()



import os
import sam_pytools
from ...del_migrations import del_process

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand

try:
    import psycopg2
    from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
except:
    pass


class Command(BaseCommand):
    help = 'setting up db i.e. create db or drop db for dev purpose'
    str = 'website/management/commands'

    def drop_create_db(self):
        res = 'Unknown'
        database_info = settings.DATABASES['default']
        del_process()
        db_engine = database_info['ENGINE']
        if db_engine.endswith('sqlite3') or db_engine.endswith('sqlite3_backend'):
            db_path = str(settings.BASE_DIR) + '/db.sqlite3'
            if os.path.exists(db_path):
                os.remove(db_path)
            return 'created'
        else:
            db_con = None
            if db_engine.endswith('postgresql') or db_engine.endswith('postgresql_backend'):
                db_con = psycopg2.connect(
                    host="localhost", user=database_info['USER'],
                    dbname='postgres', password=database_info['PASSWORD']
                )
                db_con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = db_con.cursor()
                disconnect_query = f"""
                SELECT pg_terminate_backend(pg_stat_activity.pid)
                FROM pg_stat_activity
                WHERE pg_stat_activity.datname = '{database_info['NAME']}'
                  AND pid <> pg_backend_pid();
                """
                cur.execute(disconnect_query)
                drop_query = f"DROP DATABASE IF EXISTS {database_info['NAME']}"
                cur.execute(drop_query)
                create_query = f"CREATE DATABASE {database_info['NAME']}"
                cur.execute(create_query)
            if db_con:
                db_con.close()
                return 'created'
            else:
                return ' failed to connect'

    def add_arguments(self, parser):
        parser.add_argument(
            '-hard', '--hard', action='store_true',
            help='drop database if exists and create new one'
        )

    def handle(self, *args, **kwargs):
        try:
            res = self.drop_create_db()
            if res == 'created':
                call_command('makemigrations')
                call_command('migrate')
                fixture_path = settings.FIXTURES_PATH if hasattr(settings, 'FIXTURES_PATH') else ''
                if fixture_path and os.path.isfile(fixture_path):
                    call_command('loaddata', fixture_path)
                else:
                    print(f'{fixture_path} does not exist')
            elif res == 'already exists':
                print('Already created')
            else:
                print(f'Failed because {res}')
        except:
            error_message = sam_pytools.LogUtils.get_error_message()
            print(f'Error {error_message}')



from django.conf import settings
from django.apps import AppConfig


class SamToolsConfig(AppConfig):
    name = 'sam_djtools'

    def ready(self):
        settings.MIDDLEWARE = ['django_currentuser.middleware.ThreadLocalUserMiddleware'] + settings.MIDDLEWARE



from django.db import models
from sam_djtools.admin_models import SamModel


class ModelWithUpdateLog(SamModel):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name



from datetime import datetime

import pytz
from django.db import models
from django.contrib import admin
from django.conf import settings

from django.contrib.auth.models import User as UserModel
from django_currentuser.middleware import get_current_user

def get_default_user():
    cur_user = get_current_user()
    if not cur_user:
        raise Exception('No User found in request')
    return cur_user

def get_time_now():
    return datetime.now(tz=pytz.UTC)


class SamModel(models.Model):
    class Meta:
        abstract = True

    created_at = models.DateTimeField(blank=True)
    updated_at = models.DateTimeField(blank=True)

    created_by = models.ForeignKey(
        UserModel,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_created_by",
        default=get_default_user
    )

    updated_by = models.ForeignKey(
        UserModel,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_updated_by",
        default=get_default_user
    )

    def save(self, *args, **kwargs):
        if not self.pk:
            self.created_at = self.created_at or datetime.now(tz=pytz.UTC)
            self.created_by = self.created_by or get_current_user()
        self.updated_at = datetime.now(tz=pytz.UTC)
        self.updated_by = get_current_user()
        super().save(*args, **kwargs)


log_fields = ['created_at', 'created_by', 'updated_at', 'updated_by']
aul_is = hasattr(settings, 'ADMIN_UPDATE_LOGS')
from .utils import GeneralUtils
class SamAdmin(admin.ModelAdmin):
    readonly_fields = log_fields if aul_is and settings.ADMIN_UPDATE_LOGS else []
    save_on_top = settings.SAVE_ON_TOP if hasattr(settings, 'SAVE_ON_TOP') else False
    exclude = log_fields if not aul_is or (aul_is and not settings.ADMIN_UPDATE_LOGS) else []

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        context['site_url'] = GeneralUtils.site_url(request)
        res = super().render_change_form(request, context, add, change, form_url)
        return res


class SamInlineAdmin(admin.StackedInline):
    exclude = log_fields



from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-5)7nlk&^8s667jz8z3xwagreb@dv-ls5hqizr%3nn@^z3#neq7'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'main_app.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'main_app.wsgi.application'


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}


AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

import os

my_apps = ['sam_djtools', 'sample_app']
INSTALLED_APPS = my_apps + INSTALLED_APPS
# INSTALLED_APPS = INSTALLED_APPS + my_apps
ALLOW_UNICODE_SLUGS = True


PATH_PREFIX = ''
slashed_path = ''
if PATH_PREFIX:
    slashed_path = '/' + PATH_PREFIX
if not slashed_path.endswith('/'):
    slashed_path += '/'

STATIC_ROOT = os.path.join(BASE_DIR, "static/")
MEDIA_ROOT = os.path.join(BASE_DIR, "media/")

MEDIA_URL = slashed_path + 'media/'
STATIC_URL = slashed_path + 'static/'
TIME_ZONE = 'UTC'
USE_TZ = True
ADMIN_UPDATE_LOGS = True
FIXTURES_PATH = str(BASE_DIR) + '/fixtures.json'



import base64
import threading
from urllib.parse import unquote
from sam_pytools import LogUtils

from django.apps import apps
from django.db.models import Q
from django.conf import settings
from django.db import connection
from django.core.mail import send_mail
from django.core.paginator import Paginator
from django.forms.models import model_to_dict
from django.core.files.base import ContentFile
from django.template.loader import render_to_string
default_app_config = 'sam_djtools.apps.SamToolsConfig'


class EmailUtils:

    @classmethod
    def send_text_emails_asynchronous(cls, recipient_emails, subject, message):
        cls.send_email_thread(recipient_emails, subject, message)

    @classmethod
    def send_template_emails_asynchronous(cls, recipient_emails, subject, template_name, data_dict):
        html_message = render_to_string(data_dict, template_name)
        cls.send_email_thread(recipient_emails, subject, html_message)

    @classmethod
    def send_email_thread(cls, receiver_emails, subject, html_message, sender_email="sami@gmail.com"):

        def sender_method():
            try:
                send_mail(
                    subject, "", recipient_list=receiver_emails, from_email=sender_email,
                    fail_silently=False, html_message=html_message
                )
                print('Email sent to ')
                print(receiver_emails)
            except:
                email_status = LogUtils.get_error_message()
                print('Email sending failed => '+email_status)

        thread = threading.Thread(target=sender_method)
        thread.start()


class DbUtils:

    @classmethod
    def search_db(cls, params, search_fields=None):
        results = None
        search_text = params['kw'].lower()
        search_models = params.get('search_models')
        param_search_apps = params.get('search_apps')
        if not param_search_apps:
            if hasattr(settings, 'SEARCH_APPS'):
                param_search_apps = settings.SEARCH_APPS
        search_apps = param_search_apps or {}

        for app_name in search_models:
            for model_name in search_models[app_name]:
                if not search_apps.get(app_name):
                    raise 'App '+ app_name+' not found in search_apps'
                fields = search_fields or search_apps[app_name][model_name]
                kwargs = {}
                q_objects = Q()
                for field in fields:
                    q_objects |= Q(**{field + '__contains': params['kw']})
                    kwargs.update({'{0}__{1}'.format(field, 'contains'): search_text})
                model_obj = apps.get_model(app_name, model_name)
                search_result = model_obj.objects.filter(q_objects).order_by('-pk')
                results = search_result
        return results

    @classmethod
    def execute_update(cls, query, params=None):
        cr = connection.cursor()
        if params:
            cr.execute(query, params)
        else:
            cr.execute(query)
        res = cr.rowcount
        return res

    @classmethod
    def execute_query(cls, query, params=None):
        cr = connection.cursor()
        if params:
            cr.execute(query, params)
        else:
            cr.execute(query)
        rows = cr.fetchall()
        return rows

    @classmethod
    def dict_fetch_all(cls, query, params=None):
        cr = connection.cursor()
        if params:
            cr.execute(query, params)
        else:
            cr.execute(query)
        desc = cr.description
        res = [row for row in cr.fetchall()]
        dict_list = [dict(zip([col[0].replace('.', '=>') for col in desc], row)) for row in res]
        return dict_list

    @classmethod
    def get_fields_for_tables(cls, tables):
        table_names = "','".join(tables)
        db_name = connection.settings_dict['NAME']
        query = "select concat(table_name,'.',column_name) from information_schema.columns"
        query += f" where table_schema='{db_name}' and table_name in ('{table_names}')"
        res = cls.execute_query(query)
        columns = []
        for item in res:
            columns.append(item[0])
        return columns

    @classmethod
    def filtered_list(cls, list_objects, params):
        search_kw = params.get('search_kw')
        show_fields = params.get('fields') or ''
        search_fields = params.get('search_fields')
        sort_by = params.get('sort_by')
        page_no = 0
        page_size = params.get('page_size')
        if page_size:
            page_size = int(page_size)
            page_no = params.get('page_no') or 1
            if page_no:
                page_no = int(page_no)
        args = Q()
        if search_fields:
            args_dict = {}
            arr = search_fields.split(',')
            for item in arr:
                args_dict[item+'__icontains'] = search_kw
            args.add(Q(**args_dict), Q.OR)
        list_data1 = list_objects.filter(args).distinct()
        pagination = {}
        if sort_by:
            sort_by = sort_by.split(',')
            list_data1 = list_data1.order_by(*sort_by)
        if page_size:
            page_no = page_no or 1
            if page_no < 1:
                page_no = 1
            paginator = Paginator(list_data1, page_size)
            page_count = paginator.num_pages
            if page_no >= page_count:
                page_no = page_count
            records_on_page = paginator.per_page
            if page_no == page_count:
                records_on_page = paginator.count % paginator.per_page

            list_data1 = paginator.page(page_no)
            pagination = {
                'num_pages': paginator.num_pages,
                'count': paginator.count,
                'page_number': page_no,
                'per_page': paginator.per_page,
                'records_on_page': records_on_page
            }
            list_data1 = list_data1.object_list
        if show_fields:
            show_fields = show_fields.split(',')
            list_data1 = list_data1.values(*show_fields)
        else:
            list_data1 = list_data1.values()
        list_data1 = cls.serialize_list(list_data1)
        return list_data1, pagination

    @classmethod
    def serialize_object(cls, item):
        dict_obj = {}
        for key in item:
            attr = item[key]
            dict_obj[key] = attr
        return dict_obj

    @classmethod
    def serialize_list(cls, data):
        res = []
        for item in data:
            res.append(cls.serialize_object(item))
        return res


class GeneralUtils:

    @classmethod
    def queryset_to_list(cls, queryset, fields=None, to_str=None, related=None):
        obj_list = []
        for obj in queryset:
            dict_obj = cls.obj_to_dict(obj, fields, to_str, related)
            obj_list.append(dict_obj)
        return obj_list

    @classmethod
    def obj_to_dict(cls, obj, fields=None, to_str=None, related=None):
        if fields:
            dict_obj = model_to_dict(obj, fields)
            for field in fields:
                if field.find("__") != -1:
                    val = getattr(obj, field.split("__")[0])
                    if val:
                        val = getattr(val, field.split("__")[1])
                    dict_obj[field] = val
        else:
            dict_obj = model_to_dict(obj)

        for field in dict_obj:
            # handled non url file fields (saved as binary string)
            if type(dict_obj[field]) is not str and type(dict_obj[field]) is not int:
                if dict_obj[field]:
                    if str(type(dict_obj[field])) in ["<class \'datetime.datetime\'>", "<class 'datetime.date'>"]:
                        dict_obj[field] = str(dict_obj[field])
                    elif str(type(dict_obj[field])) in ["<class \'django.db.models.fields.files.FieldFile\'>",
                                                    '<class \'django.db.models.fields.files.ImageFieldFile\'>']:
                        try:
                            file_url = dict_obj[field].url
                            if not file_url:
                                dict_obj[field] = str(dict_obj[field])
                            else:
                                dict_obj[field] = file_url
                        except:
                            if dict_obj[field].startswith('/media/data'):
                                dict_obj[field] = dict_obj[field][7:]
                                dict_obj[field] = unquote(dict_obj[field])
                else:
                    dict_obj[field] = None
        if to_str:
            for field in to_str:
                if dict_obj[field]:
                    dict_obj[field] = str(dict_obj[field])

        if related:
            for field in related:
                _to_str = related[field].get("to_str")
                _fields = related[field].get("fields")
                _related = related[field].get("related")
                rel_obj = getattr(obj, field)
                if rel_obj._queryset_class:
                    dict_obj[field] = cls.queryset_to_list(rel_obj.filter(), fields=_fields, to_str=_to_str, related=_related)

        return dict_obj


    @classmethod
    def get_user_name(cls, user):
        name = False
        if user.first_name:
            name = user.first_name
            if user.last_name:
                name += ' ' + user.last_name
        elif user.last_name:
            name += user.last_name
        else:
            name = user.username
        return name

    @classmethod
    def base64_str_to_file(cls, data, file_name):
        header, data = data.split(';base64,')
        decoded_file = base64.b64decode(data)
        return ContentFile(decoded_file, name=file_name)

    @classmethod
    def site_url(cls, request):
        host_url = f'{request.scheme}://{request.get_host()}'
        return host_url

    @classmethod
    def full_url(cls, request):
        return request.build_absolute_uri()



from django.conf import settings
from django.db import transaction, router
from django.contrib import admin, messages
from django.http import HttpResponseRedirect


def next_prev_id(model, row_id):
    output = {"prev": "", "next": "", "current": row_id}
    rec_count = model.objects.all().count()

    if rec_count <= 1:
        return output

    prev_obj = model.objects.filter(pk__lt=row_id).order_by("-pk").first()
    next_obj = model.objects.filter(pk__gt=row_id).order_by("pk").first()

    def update_output(nav_obj, direction):
        if nav_obj:
            output[direction] = nav_obj.pk
        else:
            sign = '-' if direction == 'prev' else ''
            nav_obj = model.objects.order_by(sign  + "pk").first()
            if nav_obj and nav_obj.pk != row_id:
                output[direction] = nav_obj.pk

    update_output(prev_obj, 'prev')
    update_output(next_obj, 'next')
    return output


class NavigateFormAdmin(admin.ModelAdmin):
    save_on_top = settings.SAVE_ON_TOP if hasattr(settings, 'SAVE_ON_TOP') else False

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        try:
            if object_id:
                obj_id = int(object_id)
                ids = next_prev_id(self.model, obj_id)
                if extra_context:
                    extra_context['npc_ids'] = ids
                else:
                    extra_context = {'npc_ids': ids}
            with transaction.atomic(using=router.db_for_write(self.model)):
                return self._changeform_view(request, object_id, form_url, extra_context)
        except Exception as e:
            self.message_user(request, e, level=messages.ERROR)
            return HttpResponseRedirect(request.path)



import os
import glob
from os import path
from pathlib import Path
from django.conf import settings


def del_process():
    project_root = str(settings.BASE_DIR)
    def remove_migrations():
    
        res = glob.glob(project_root+'/**/migrations/*', recursive=True)
        skips = [project_root +'/django/']
        for file_path in res:
            for pth in skips:
                if file_path.startswith(pth):
                    continue
            if file_path.endswith('__pycache__'):
                file_path = file_path + '/*'
                sub_res = glob.glob(file_path)
                for file_path1 in sub_res:
                    os.remove(file_path1)
            elif not file_path.endswith('__init__.py'):
                if path.isfile(file_path):
                    os.remove(file_path)
        if path.exists('db.sqlite3'):
            os.remove('db.sqlite3')
        print('Migration files removed')
    
    
    def remove_file_by_extension(ext):
        cnt = 0
        files = Path(project_root).rglob('*.'+ext)
        for path in files:
            os.remove(str(path))
            cnt += 1
        print(f'{str(cnt)} {ext} files removed')
    
    
    remove_migrations()
    remove_file_by_extension('pyc')
    # remove_file_by_extension('po')
    print('done')
    



from setuptools import find_namespace_packages, setup

setup(
    name='sam-djtools',
    long_description_content_type="text/markdown",
    url="https://github.com/humblesami/sam-djtools.git",
    python_requires=">=3",
    setup_requires=['setuptools_scm'],

    include_package_data=True,
    packages=find_namespace_packages(include=[
        "sam_djtools", "sam_djtools.admin_models",
        "sam_djtools.utils", "sam_djtools.del_migrations",
    ]),
    package_data={
        "sam_djtools": [
            "navigate_records.py",
            "management/commands/*",
            "templates/admin/change_form.html",
            "static/sam_djtools/change_form_prev_next.js"
        ]
    }
)



from django.contrib import admin
from sam_djtools.admin_models import SamAdmin
from .models import ModelWithUpdateLog


class ModelUpdateLogAdmin(SamAdmin):
    pass


# Register your models here.
admin.site.register(ModelWithUpdateLog, ModelUpdateLogAdmin)



import os
import glob
from pathlib import Path
dir_path = str(os.path.dirname(os.path.realpath(__file__)))


def remove_migrations():

    res = glob.glob(dir_path+'/**/migrations/*', recursive=True)
    skips = [str(dir_path) +'/django/']
    for file_path in res:
        for pth in skips:
            if file_path.startswith(pth):
                continue
        if file_path.endswith('__pycache__'):
            file_path = file_path+'/*'
            sub_res = glob.glob(file_path)
            for file_path1 in sub_res:
                os.remove(file_path1)
        elif not file_path.endswith('__init__.py'):
            if os.path.isfile(file_path):
                os.remove(file_path)
    if os.path.exists('db.sqlite3'):
        os.remove('db.sqlite3')
    print('Migration files removed')


def remove_file_by_extension(ext):
    cnt = 0
    files = Path(dir_path).rglob('*.'+ext)
    for path in files:
        os.remove(str(path))
        cnt += 1
    print(str(cnt) + ' ' + ext + ' files removed')


remove_migrations()
remove_file_by_extension('pyc')
remove_file_by_extension('po')
print('done')

